<?php 

	class AdvOrderS_Lite_Orders_Settings extends WC_Integration  {

		function __construct() {

			add_action( 'woocommerce_update_options_integration_wc_lite_orderstatus', array( &$this, 'process_admin_options') );

			add_action( 'admin_enqueue_scripts', array( &$this, 'admin_head' ) );

			$this->id = 'wc_lite_orderstatus';

			$this->method_title = __( 'AppZab Advance Order Status', 'advorder_lite_orderstatus' );

			$this->method_description = __( 'These settings control custom order statuses', 'advorder_lite_orderstatus' );

			$this->init_form_fields();

			$this->init_settings();

			foreach( $this->settings as $k => $v )
				unset( $this->settings[$k] );

			$this->enabled = false;

			add_action( 'init', array( &$this, 'maybe_adjust_statuses' ), 9999999 );

		}


		function admin_head() {
			global $wp_version;

			if ( isset( $_GET['section'] ) && 'wc_lite_orderstatus' != $_GET['section'] )
				return;

			if ( version_compare( $wp_version, '3.5' ) < 0 ) {

				wp_enqueue_style( 'farbtastic' );
				
				wp_enqueue_script( 'farbtastic' );
				

			} else {
			
				wp_enqueue_script( 'wp-color-picker' );
				
				wp_enqueue_style( 'wp-color-picker' );
			}


			
		}
		
		function maybe_adjust_statuses() {

			if ( !is_admin() )
				return;

			if ( !isset( $_REQUEST['advorder_lite_orderstatus'] ) )
				return;

			if ( isset( $_GET['edit_status'] ) && !isset( $_POST['edit_status_id'] ) )
				return;

			if ( isset( $_GET['page'] ) && 'woocommerce_settings' == $_GET['page'] )
				$url = admin_url( 'admin.php?page=woocommerce_settings&tab=integration&section=wc_lite_orderstatus&appzab_lite_orderstatus_message=' );
			else
				$url = admin_url( 'admin.php?page=wc-settings&tab=integration&section=wc_lite_orderstatus&appzab_lite_orderstatus_message=' );

			if ( !empty( $_POST ) && ( !isset( $_POST['woocommerce_wc_lite_orderstatus_name'] ) || '' == trim( $_POST['woocommerce_wc_lite_orderstatus_name'] ) ) ) {
				
				wp_redirect( $url . 'empty' );

				die;

			}

			if ( isset( $_POST['edit_status_id'] ) && absint( $_POST['edit_status_id'] ) > 0 ) {

				if ( !isset( $_POST['woocommerce_wc_lite_orderstatus_name'] ) || '' == trim( $_POST['woocommerce_wc_lite_orderstatus_name'] ) ) {
				
					wp_redirect( $url . 'empty' );

					die;

				}

				
				$args = array( 'name' => $_POST['woocommerce_wc_lite_orderstatus_name'] );
			
				wp_update_term( absint( $_POST['edit_status_id'] ), 'shop_order_status', $args );

				$this->save_status_custom_meta( absint( $_POST['edit_status_id'] ) );

				wp_redirect( $url . 'updated' );

				die;


			}
			
			if ( isset( $_GET['remove'] ) && absint( $_GET['remove'] ) > 0 ) {

				wp_delete_term( absint( $_GET['remove'] ),'shop_order_status' );

				delete_option( 'taxonomy_' . absint( $_GET['remove'] ) );

				wp_redirect( $url . 'removed' );

				die;


			}
		
		
			$term_id = wp_insert_term( $_POST['woocommerce_wc_lite_orderstatus_name'], 'shop_order_status' );

			if ( !is_wp_error( $term_id ) && $term_id['term_id'] > 0 ) {

				$this->save_status_custom_meta( $term_id['term_id'] );
				
				wp_redirect( $url . 'success' );

				die;

			} else {

				if ( is_wp_error( $term_id ) )
				foreach( $term_id->errors as $k => $v  ) {

					$err = urlencode( $v[0] );
					
					wp_redirect( $url . 'fail&error=' . $err );

					die;
					
				}
				
				wp_redirect( $url . 'fail&error=' );

				die;

			}

		}


		function save_status_custom_meta( $term_id ) {

			$meta = array();

			foreach( $_POST as $k => $v ) {

				if ( false !== strpos( $k, 'woocommerce_wc_lite_orderstatus' ) )
					$meta[ $k ] = $v;

			}

			update_option( 'taxonomy_' . $term_id, $meta );

		}


		function init_form_fields() {

			$this->form_fields = array(
					'name' => array(
						'title' 	=> __( 'Status Name', 'advorder_lite_orderstatus' ),
						'description' 	=> __( 'Enter the name of custom order status', 'advorder_lite_orderstatus' ),
						'type' 		=> 'text',
						'default' 	=> '',
					),
					'bcolor' => array(
						'title' 	=> __( 'Background Color', 'advorder_lite_orderstatus' ),
						'description' 	=> __( '<span class="colorpickdiv" style="display:none"></span>Enter the hex color code to use when displaying this status in the orders page ', 'advorder_lite_orderstatus' ),
						'type' 		=> 'text',
						'tip' 		=> 'asdf',
						'default' 	=> '',
						'class' 	=> ' colorpick '
					),
					'fcolor' => array(
						'title' 	=> __( 'Font Color', 'advorder_lite_orderstatus' ),
						'description' 	=> __( '<span class="colorpickdiv" style="display:none"></span>Enter the hex color code to use when displaying this status in the orders page ', 'advorder_lite_orderstatus' ),
						'type' 		=> 'text',
						'tip' 		=> 'asdf',
						'default' 	=> '#ffffff',
						'class' 	=> ' colorpick '
					),
					'reports' => array(
						'title' 	=> __( 'Add to Reports', 'advorder_lite_orderstatus' ),
						'description' 	=> __( 'Add to Sales Reports', 'advorder_lite_orderstatus' ),
						'label' 	=> 'Enable',
						'type' 		=> 'checkbox',
						'default' 	=> 'no'
					),
					'email' => array(
						'title' 	=> __( 'Send Email', 'advorder_lite_orderstatus' ),
						'description' 	=> __( 'Send the message below when an order is set to this status', 'advorder_lite_orderstatus' ),
						'label' 	=> 'Enable',
						'type' 		=> 'checkbox',
						'default' 	=> 'no'
					),
					'subject' => array(
						'title' 	=> __( 'Subject', 'advorder_lite_orderstatus' ),
						'description' 	=> __( 'The message subject', 'advorder_lite_orderstatus' ),
						'type' 		=> 'text',
						'default' 	=> ''
					),
					'heading' => array(
						'title' 	=> __( 'Heading', 'advorder_lite_orderstatus' ),
						'description' 	=> __( 'The heading inserted at the top of the message', 'advorder_lite_orderstatus' ),
						'type' 		=> 'text',
						'tip' 		=> 'asdf',
						'default' 	=> ''
					),
					'message' => array(
						'title' 	=> __( 'Message', 'advorder_lite_orderstatus' ),
						'description' 	=> __( 'HTML Allowed. Yay!', 'advorder_lite_orderstatus' ),
						'type' 		=> 'textarea',
						'default' 	=> ''
					),
					'send_to' => array(
						'title' 	=> __( 'Send To', 'advorder_lite_orderstatus' ),
						'description' 	=> __( 'Enter the email address where you want a notification to be sent. You can leave it blank to send it to the customer', 'advorder_lite_orderstatus' ),
						'type' 		=> 'text',
						'tip' 		=> 'asdf',
						'default' 	=> ''
					),
			);
		}


		function excerpt( $content, $maxchars = 125 ) {
		
			$content = substr( stripslashes( $content ), 0, $maxchars );

			$pos = strrpos( $content, " " );

			if ( $pos > 0 )
				$content = substr( $content, 0, $pos );
			
			echo $content;

		}

		
		function admin_options() {
			global $woocommerce, $wpdb, $wp_version;
			
			$defaults = array( 'pending','failed','on-hold','processing','completed','refunded','cancelled' );

			$statuses = (array) $GLOBALS['advorder_lite_orderstatus']->get_terms( 'shop_order_status', array( 'hide_empty' => 0, 'orderby' => 'id' ) );
			
			if (  isset( $_GET['appzab_lite_orderstatus_message'] ) ) { 
			
			if ( 'fail' == $_GET['appzab_lite_orderstatus_message'] ) {
				?>
				
				<div class="error">
					<p>
						<?php _e( '*** STATUS NOT ADDED ***','advorder_lite_orderstatus' ); ?>

						<?php
						if ( isset( $_GET['error'] ) ) {
						
							_e( 'ERROR: ','advorder_lite_orderstatus' );
							
							echo $_GET['error'];
						}
						?>
					</p>
				</div>

				<?php
				
			} else if ( 'empty' == $_GET['appzab_lite_orderstatus_message'] ) {
				?>
				
				<div class="error">
					<p>
						<?php _e( 'You must enter a name for the status','advorder_lite_orderstatus' ); ?>
					</p>
				</div>

				<?php

			} else if ( 'success' == $_GET['appzab_lite_orderstatus_message'] ) {
				?>
				
				<div class="updated fade">
					<p>
						<?php _e( 'New status added','advorder_lite_orderstatus' ); ?>
					</p>
				</div>

				<?php
				
			} else if ( 'removed' == $_GET['appzab_lite_orderstatus_message'] ) {
				?>

				<div class="updated fade">
					<p>
						<?php _e( 'The status has been removed.','advorder_lite_orderstatus' ); ?>
					</p>
				</div>

				<?php
			} else if ( 'update' == $_GET['appzab_lite_orderstatus_message'] ) {
				?>

				<div class="updated fade">
					<p>
						<?php _e( 'Status updated.','advorder_lite_orderstatus' ); ?>
					</p>
				</div>

				<?php
			}
			}
			$this->settings = array();

			$editing = false;

			$button_string = __( 'Add New Status', 'advorder_lite_orderstatus' );
			$header_string = __( 'Add New Status', 'advorder_lite_orderstatus' );

			if ( isset( $_GET['edit_status'] ) && absint( $_GET['edit_status'] ) > 0 ) {

				$term = get_term_by( 'id', absint( $_GET['edit_status'] ), 'shop_order_status' );

				if ( !is_wp_error( $term ) ) {

					$term_meta = get_option( 'taxonomy_' . $term->term_id, false );

					$this->settings['name'] = $term->name;
					
					if ( $term_meta ) {

						$this->settings['fcolor'] = $term_meta['woocommerce_wc_lite_orderstatus_fcolor'];
						
						$this->settings['bcolor'] = $term_meta['woocommerce_wc_lite_orderstatus_bcolor'];

						if ( isset( $term_meta['woocommerce_wc_lite_orderstatus_email'] ) && '1' == $term_meta['woocommerce_wc_lite_orderstatus_email'] ) 
							$this->settings['email'] = 'yes';
						else
							$this->settings['email'] = 'no';

						if ( isset( $term_meta['woocommerce_wc_lite_orderstatus_reports'] ) && '1' == $term_meta['woocommerce_wc_lite_orderstatus_reports'] ) 
							$this->settings['reports'] = 'yes';
						else
							$this->settings['reports'] = 'no';
							
						$this->settings['subject'] = stripslashes( $term_meta['woocommerce_wc_lite_orderstatus_subject'] );

						$this->settings['message'] = stripslashes( $term_meta['woocommerce_wc_lite_orderstatus_message'] );

						$this->settings['heading'] = stripslashes( $term_meta['woocommerce_wc_lite_orderstatus_heading'] );

						$this->settings['send_to'] = stripslashes( $term_meta['woocommerce_wc_lite_orderstatus_send_to'] );
					}
					
					$editing = true;
					
					$button_string = __( 'Update Status', 'advorder_lite_orderstatus' );

					$header_string = __( 'Editing Status', 'advorder_lite_orderstatus' );
				}

			}

			?>
			<style>
			.status_col mark { 
				border-radius: 3px 3px 3px 3px;
				display: block;
				font-size: 11px;
				font-weight: bold;
				line-height: 1.75;
				margin: 0;
				padding: 0 2px;
				text-align: center;
				white-space: nowrap;
				width: 70px;
			}
			.widefat th {
				font-weight: bold;
			}
			.widefat thead, .widefat tbody { 
				background: #fff !important;
			}
			.form-table td, .form-table th {
				padding: 1.5em;
			}
			</style>
			<script>
				jQuery.extend( jQuery.fn, {
					hasParent: function(p) {
						return this.filter(function(){
							return jQuery(p).find(this).length;
						});
					}
				});
				jQuery( document ).ready( function() {
				
					jQuery( ".submit" ).find( "input[type='submit']").val( "<?php echo $button_string ?>" );

					jQuery( ".status_remove" ).click( function() {

						if ( !confirm( '<?php _e( 'Are you sure you want to remove this status?', 'advorder_lite_orderstatus' ) ?>' ) )
							return false;

					})

					<?php if ( version_compare( $wp_version, '3.5' ) < 0 ) { ?>

						// Color picker
						jQuery('.colorpick').each(function(){
							jQuery('.colorpickdiv', jQuery(this).parent()).farbtastic(this);
							jQuery(this).click(function() {
								if ( jQuery(this).val() == "" ) jQuery(this).val('#');
								jQuery('.colorpickdiv', jQuery(this).parent() ).show();
							});
						});
						jQuery(document).mousedown(function(){
							jQuery('.colorpickdiv').hide();
						});

					<?php  } else { ?>

							jQuery( ".colorpick" ).wpColorPicker();

							jQuery( document ).mousedown( function( e ) {

								if ( jQuery( e.target ).hasParent( ".wp-picker-holder" ) )
									return;

								if ( jQuery( e.target ).hasParent( "mark" ) )
									return;

								jQuery( ".wp-picker-holder" ).each(function() {
									jQuery( this ).fadeOut();
								});
							});	 
					<?php } ?>
				
				});
				
			</script>
			
			<h3><?php echo ( ! empty( $this->method_title ) ) ? $this->method_title : __( 'Settings','advorder_lite_orderstatus' ) ; ?></h3>

			<p><em><?php _e( 'Only statuses created with this plugin have a "Sends Email" indicator', 'advorder_lite_orderstatus' )?></em></p>

			<table class="widefat" style="width:auto;  display:inline; clear:none;">

				<thead>
					<tr>
						<th style="min-width:30px;"><?php _e( 'ID','advorder_lite_orderstatus' ); ?></th>

						<th style="min-width:80px;"><?php _e( 'Name','advorder_lite_orderstatus' ); ?></th>
						
						<th><?php _e( 'Slug','advorder_lite_orderstatus' ); ?></th>

						<th><?php _e( 'Sends Email','advorder_lite_orderstatus' ); ?></th>

						<th><?php _e( 'Subject','advorder_lite_orderstatus' ); ?></th>

						<th><?php _e( 'Header','advorder_lite_orderstatus' ); ?></th>

						<th><?php _e( 'Excerpt','advorder_lite_orderstatus' ); ?></th>

						<th><?php _e( 'Send To','advorder_lite_orderstatus' ); ?></th>
						
						<th><?php _e( 'Actions','advorder_lite_orderstatus' ); ?></th>
					</tr>
				</thead>

			<?php
				foreach( $statuses as $status) {

					$meta = get_option( 'taxonomy_' . $status->term_id, false );

					if ( !$meta || empty( $meta['woocommerce_wc_lite_orderstatus_fcolor'] ) )
						$fgcolor = '#000000';
					else
						$fgcolor = $meta['woocommerce_wc_lite_orderstatus_fcolor'];
				
					if ( !$meta || empty( $meta['woocommerce_wc_lite_orderstatus_bcolor'] ) )
						$bgcolor = '#ffffff';
					else
						$bgcolor = $meta['woocommerce_wc_lite_orderstatus_bcolor'];
					
					if ( !$meta )
						$sends_email = '';
						
					else if ( $meta && isset( $meta['woocommerce_wc_lite_orderstatus_email'] ) && ( '1' == $meta['woocommerce_wc_lite_orderstatus_email'] || 'yes' == $meta['woocommerce_wc_lite_orderstatus_email'] ) )
						$sends_email = 'Yes';
						
					else
						$sends_email = 'No';
					?>


					
					<tr <?php if ( !in_array( $status->slug, $defaults ) ) {} ?>>
					
						<td><?php echo $status->term_id ?></td>

						<td class="status_col">
							<mark style="background-color: <?php echo $bgcolor ?>; color: <?php echo $fgcolor ?>; <?php if ( in_array( $status->name, $defaults ) ) echo ' font-weight:normal' ?>" ><?php echo $status->name ?></mark>
						</td>

						<td><?php echo $status->slug ?></td>

						<td><?php echo $sends_email ?></td>

						<td>
							<?php 
							if ( isset( $meta['woocommerce_wc_lite_orderstatus_subject'] ) ) {
								echo $meta['woocommerce_wc_lite_orderstatus_subject']; 
							} 
							?>
						</td>

						<td>
							<?php 
							if ( isset( $meta['woocommerce_wc_lite_orderstatus_header'] ) ) {
								echo $meta['woocommerce_wc_lite_orderstatus_header'];
							} ?>
						</td>

						<td>
							<?php
							if ( isset( $meta['woocommerce_wc_lite_orderstatus_message'] ) ) { 
								echo $this->excerpt( $meta['woocommerce_wc_lite_orderstatus_message'], 125 );
							}
							?>
						</td>
							<?php //if ( $meta['woocommerce_wc_lite_orderstatus_message'] == strip_tags( $meta['woocommerce_wc_lite_orderstatus_message'] ) )
								//echo $this->excerpt( $meta['woocommerce_wc_lite_orderstatus_message'], 125 ) 
							?>
						<td><?php echo $meta['woocommerce_wc_lite_orderstatus_send_to']  ?></td>

						<td>
					<?php
					
						if ( !in_array( $status->slug, $defaults ) ) {

							if ( isset( $_GET['page'] ) && 'woocommerce_settings' == $_GET['page'] )
								$url = admin_url( 'admin.php?page=woocommerce_settings&tab=integration&section=wc_lite_orderstatus&advorder_lite_orderstatus=1&remove=' . $status->term_id );
							else
								$url = admin_url( 'admin.php?page=wc-settings&tab=integration&section=wc_lite_orderstatus&advorder_lite_orderstatus=1&remove=' . $status->term_id );
						
							echo '<a class="status_remove" href="'. $url .'">'.__( 'Remove','advorder_lite_orderstatus' ).'</a>';

							echo ' | ';

							if ( isset( $_GET['page'] ) && 'woocommerce_settings' == $_GET['page'] )
								$url = admin_url( 'admin.php?page=woocommerce_settings&tab=integration&section=wc_lite_orderstatus&advorder_lite_orderstatus=1&edit_status=' . $status->term_id . '#add_edit' );
							else
								$url = admin_url( 'admin.php?page=wc-settings&tab=integration&section=wc_lite_orderstatus&advorder_lite_orderstatus=1&edit_status=' . $status->term_id . '#add_edit' );
							
							echo '<a href="'. $url .'">'.__( 'Edit','advorder_lite_orderstatus' ).'</a>';

						} else {

							echo '<span style="font-style:italic;">'.__( 'WooCommerce Default','advorder_lite_orderstatus' ).'</span>';

						}
					?>

						</td>
					</tr>
				<?php 
				}
			?>

			</table>

			<?php if ( $editing ) { ?>

				<p style="font-style:italic; color: #cf0000"><?php _e( 'You can edit the name, but not the slug. All slugs must be unique and are automatically assigned by this plugin.', 'advorder_lite_orderstatus' ) ?></p>

				<input type="hidden" name="edit_status_id" value="<?php echo absint( $_GET['edit_status'] ) ?>">
				
			<?php } ?>

			<a name="add_edit">&nbsp;</a>

			<table class="form-table" style="clear:both; margin-top: 20px; border: 1px solid #ccc" >

				<thead>
					<tr>
						<th colspan="2" style="background-color:#eee">
							<strong><?php echo $header_string ?></strong>
						</th>
					</tr>
					<tr>
						<th colspan="2"><em>
						<?php _e( 'Easy shortcodes: %o = Order ID, %bf = Billing First Name, %bl = Billing Last Name, %sf = Shipping First Name, %sl = Shipping Last Name, %s = Site Name, %u = Site URL, %m = My Account page URL','advorder_lite_orderstatus' ) ?>
						</em></th>
					</tr>
				</thead>

				<?php $this->generate_settings_html(); ?>

			</table>

			<input type="hidden" name="advorder_lite_orderstatus" value="1">

		<?php }

	}

//}