<?php
/*
Plugin Name: WooCommerce Advance Order Status
Plugin URI: http://www.appzab.com
Description: Add custom order statuses to your WooCommerce platform with email notifications for each order status
Version: 1.1.3
Author: AppZab
Author URI: http://AppZab.com
*/


if ( !function_exists( 'appz_order_status_active' ) ) {

	function appz_order_status_active() {

		$active_plugins = (array) get_option( 'active_plugins', array() );

		if ( is_multisite() )
			$active_plugins = array_merge( $active_plugins, get_site_option( 'active_sitewide_plugins', array() ) );

		return in_array('woocommerce/woocommerce.php', $active_plugins) || array_key_exists('woocommerce/woocommerce.php', $active_plugins);

	}
}


add_action( 'plugins_loaded', 'advorder_lite_orderstatus_go', 1 );

function advorder_lite_orderstatus_go() {

	if ( !appz_order_status_active() )
		return;

	if ( appz_order_status_active() )
		$GLOBALS['advorder_lite_orderstatus'] = new Advorder_Lite_Orderstatus();

	$this_plugin_base = plugin_basename( __FILE__ );

	
}
	


class Advorder_Lite_Orderstatus {

	function __construct() {
		
		add_action( 'init', array( &$this, 'load_plugin_textdomain' ) );

		add_action( 'init', array( &$this, 'setup_status_actions' ), 99 );

		add_action( 'init', array( &$this, 'maybe_do_upgrade' ), 9999 ); // after setup_status_actions runs, so we have default statuses set
		
		add_action( 'admin_head', array( &$this, 'admin_head' ) );

		add_action( 'woocommerce_integrations', array( &$this, 'advorder_lite_orderstatus_integration' ), 999 );
		
		add_action( 'admin_footer', array( &$this, 'bulk_admin_footer' ), 10 );
		add_action( 'load-edit.php', array( &$this, 'order_bulk_action' ) );
		add_action( 'admin_notices', array( &$this, 'order_bulk_admin_notices' ) );
		
		add_filter( 'woocommerce_reports_order_statuses', array( &$this, 'maybe_add_report_statuses' ) );
		
		add_filter( 'wc_order_statuses', array( &$this, 'filter_statuses' ), 99999 );

	}
	
	function check_for_dups( $custom_statuses, $slug ) { 

		if ( array_key_exists( $slug, $custom_statuses ) ) { 
		
				while( array_key_exists( $slug, $custom_statuses ) ) { 
			
				$pos = strrpos( $slug, '_' );
		
				if ( false !== $pos ) {
				
					$int = intval( substr( $slug, $pos + 1, strlen( $slug ) ) );
				
					$int = rand( 0, 999 );
				
					$slug = substr( $slug, 0, $pos );
					
					$slug .= '_' . $int;
				
				} else {
				
					$int = 0;
			
					$slug .= '_0'; 
			
				}
				
			}

		}

		return $slug;
	
	}
	
	function maybe_do_upgrade() {
		global $wpdb;
		
	
		if ( version_compare( WOOCOMMERCE_VERSION, '2.2', '<' ) )
			return;
		
		if ( get_option( 'advorder_order_statuses_22_compat' ) )
			return;
			
		$statuses = get_terms( 'shop_order_status', array( 'hide_empty' => 0, 'orderby' => 'id' ) );
		
		if ( empty( $statuses ) || is_wp_error( $statuses ) ) {
		
				register_taxonomy( 'shop_order_status',
				apply_filters( 'woocommerce_taxonomy_objects_shop_order_status', array('shop_order') ),
				apply_filters( 'woocommerce_taxonomy_args_shop_order_status', array(
				'hierarchical' => false,
				'update_count_callback' => '_update_post_term_count',
				'show_ui' => false,
				'show_in_nav_menus' => false,
				'query_var' => is_admin(),
				'rewrite' => false,
				'public' => false
				) )
			);

			$statuses = get_terms( 'shop_order_status', array( 'hide_empty' => 0, 'orderby' => 'id' ) );
			
		}

		if ( empty( $statuses ) || is_wp_error( $statuses ) )
			return;
		
		$upgraded_statuses = array(); 
		
		foreach( $statuses as $s ) { 
		
			if ( empty( $s->slug ) )
				continue;
				
			$s->slug = str_replace( 'wc-', '', $s->slug );

			if ( in_array( $s->slug, array( 'pending', 'processing', 'completed', 'failed', 'cancelled', 'refunded', 'on-hold' ) ) )
				continue; 
	
			$meta = get_option( 'taxonomy_' . $s->term_id, false );
		
			$s->slug = substr( $s->slug, 0, 13 );
			
			$s->slug = $this->check_for_dups( $upgraded_statuses, $s->slug );
				
			$upgraded_statuses[ 'wc-' . $s->slug ] = $s->name;
			
			update_option( 'az_custom_order_status_meta_' . $s->slug, $meta ); // add custom status meta data into new storage 
		
			$sql = "
				UPDATE {$wpdb->posts} as posts
				LEFT JOIN {$wpdb->term_relationships} AS rel ON posts.ID = rel.object_ID
				LEFT JOIN {$wpdb->term_taxonomy} AS tax USING( term_taxonomy_id )
				LEFT JOIN {$wpdb->terms} AS term USING( term_id )
				SET posts.post_status = 'wc-{$s->slug}'
				WHERE posts.post_type = 'shop_order'
				AND posts.post_status = 'publish'
				AND tax.taxonomy = 'shop_order_status'
				AND	term.slug LIKE '" . $s->slug . "%';
				";
				

			$wpdb->query( $sql );
		
		}

		update_option( 'advorder_custom_order_statuses', $upgraded_statuses );
		
		update_option( 'advorder_order_statuses_22_compat', 1 );
	
	}
	
	function load_plugin_textdomain() {

		$locale = apply_filters( 'plugin_locale', get_locale(), 'advorder_lite_orderstatus' );

		load_textdomain( 'advorder_lite_orderstatus', WP_LANG_DIR.'/woocommerce/advorder_lite_orderstatus-'.$locale.'.mo' );

		$plugin_rel_path = apply_filters( 'advorder_translation_file_rel_path', dirname( plugin_basename( __FILE__ ) ) . '/languages' );

		load_plugin_textdomain( 'advorder_lite_orderstatus', false, $plugin_rel_path );

	}

	function advorder_lite_orderstatus_integration( $integrations ) {

		if ( version_compare( WOOCOMMERCE_VERSION, '2.2', '<' ) && !class_exists( 'AdvOrderS_Lite_Orders_Settings' ) ) { 
			require_once( 'includes/app-adv-order-int.php' );
		}
		
		if ( version_compare( WOOCOMMERCE_VERSION, '2.2', '>=' ) && !class_exists( 'AdvOrderS_Lite_Orders_Settings' ) ) { 
			require_once( 'includes/app-adv-order-int-alt.php' );
		}
		
		$integrations[] = 'AdvOrderS_Lite_Orders_Settings';
		
		return $integrations;
	}

	function filter_statuses( $statuses = array() ) { 

		$custom_statuses = (array) get_option( 'advorder_custom_order_statuses' );

		if ( empty( $custom_statuses ) )
			return $statuses; 

		foreach( $custom_statuses as $key => $val ) { 
		
			register_post_status( $key, array(
				'label'                     => _x( $val, 'Order status', 'advorder_lite_orderstatus' ),
				'public'                    => true,
				'exclude_from_search'       => false,
				'show_in_admin_all_list'    => true,
				'show_in_admin_status_list' => true,
				'label_count'               => _n_noop( $val . ' <span class="count">(%s)</span>', $val . ' <span class="count">(%s)</span>', 'advorder_lite_orderstatus' )
			) );
			
		}
		
		return array_merge( $statuses, $custom_statuses );
			
	}
	
	
	function get_terms( $taxo = 'shop_order_status', $args = array() ) {
	
		if ( version_compare( WOOCOMMERCE_VERSION, '2.2', '<' ) ) {
		
			return get_terms( $taxo, $args );
			
		} else if ( version_compare( WOOCOMMERCE_VERSION, '2.2', '>=' ) ) {
		
			$s = wc_get_order_statuses();

			if ( !empty( $s ) )  {
			
				// Convert array to object for backward compat
				
				$i = 1; 
				
				foreach( $s as $key => $val ) {
			
					if ( empty( $key ) || empty( $val ) )
						continue;
						
					$status = new stdClass(); 
			
					$status->term_id = $i; 
					
					$status->slug = $key;
					
					$status->name = $val;
					
					$statuses[ $i ] = $status; 

					$i++;
					
				}

				return $statuses; 
			}
			
		}
	}

	function admin_head() {

		$defaults = $this->defaults;

		$statuses = (array) $this->get_terms( 'shop_order_status', array( 'hide_empty' => 0, 'orderby' => 'id' ) );


		?>
		<style>

		<?php
		foreach( $statuses as $s ) {
		
			if ( !in_array( $s->slug, $defaults ) ) {

				if ( version_compare( WOOCOMMERCE_VERSION, '2.2', '>=' ) ) {
				
					$meta = get_option( 'az_custom_order_status_meta_' . $s->slug, array() );

				} else { 
				
					$meta = get_option( 'taxonomy_' . $s->term_id );
					
				}

				if ( !$meta || !isset( $meta['woocommerce_wc_lite_orderstatus_fcolor'] ) || empty( $meta['woocommerce_wc_lite_orderstatus_fcolor'] ) )
					$fcolor = '#fff';
				else
					$fcolor = $meta['woocommerce_wc_lite_orderstatus_fcolor'];
					
				if ( !$meta || !isset( $meta['woocommerce_wc_lite_orderstatus_bcolor'] ) || empty( $meta['woocommerce_wc_lite_orderstatus_bcolor'] ) )
					$bcolor = '#c5f9c5';
				else
					$bcolor = $meta['woocommerce_wc_lite_orderstatus_bcolor'];
				
				?>
				.widefat .column-order_status mark.<?php echo str_replace( 'wc-', '', $s->slug ) ?> {
					background-color: <?php echo $bcolor ?> !important;
					color: <?php echo $fcolor ?> !important;
					border-radius: 20px;
				}

				<?php
			
			}
		}
		?>
		
		</style>

		<?php

	}

	
	function maybe_add_report_statuses( $statuses = array() ) { 
		global $wpdb;

		$terms = $this->get_terms( 'shop_order_status' );
		
		if ( empty( $terms ) || is_wp_error( $terms ) )
			return $statuses;
	
		foreach( $terms as $t ) { 
		 
			if ( version_compare( WOOCOMMERCE_VERSION, '2.2', '>=' ) ) {
		 
				$meta = get_option( 'az_custom_order_status_meta_' . $t->slug, array() );
				
			} else { 
		 
				$meta = get_option( 'taxonomy_' . $t->term_id, false );
				
			}
			
			if ( empty( $meta ) || empty( $meta['woocommerce_wc_lite_orderstatus_reports'] ) )
				continue; 
				
			if ( '1' == absint( $meta['woocommerce_wc_lite_orderstatus_reports'] ) ) 
				$statuses[] = str_replace( 'wc-', '', $t->slug );
		
		}

		return $statuses;
	
	}
	

	function setup_status_actions() {

		if ( version_compare( WOOCOMMERCE_VERSION, '2.2', '<' ) ) { 
		
			$this->defaults = array( 'pending','failed','on-hold','processing','completed','refunded','cancelled' );
		
		} else if ( version_compare( WOOCOMMERCE_VERSION, '2.2', '>=' ) ) { 
		
			$ss = wc_get_order_statuses();
			
			$custom_statuses = get_option( 'advorder_custom_order_statuses', array() );
			
			foreach( $ss as $key => $val ) {
			
				// bypass custom statuses, we only want default WC statuses
				if ( !isset( $custom_statuses[ $key ] ) )
					$this->defaults[] = $key;
				
			}
		
			unset( $ss );
		}
		
		$statuses = (array) $this->get_terms( 'shop_order_status', array( 'hide_empty' => 0, 'orderby' => 'id' ) );

		if ( $statuses && !is_wp_error( $statuses ) )
		foreach( $statuses as $s ) {

			if ( in_array( $s->slug, $this->defaults ) )
				continue;

			if ( version_compare( WOOCOMMERCE_VERSION, '2.2', '>=' ) )
				$s->slug = str_replace( 'wc-', '', $s->slug );
				
			add_action( 'woocommerce_order_status_' . $s->slug, array( &$this, 'do_status_change' ), 1, 1 );

		}

	}


	function do_status_change( $order_id ) {
		global $woocommerce;

		if ( version_compare( WOOCOMMERCE_VERSION, '2.2', '>=' ) ) { 
		
			$this->do_status_change_alt( $order_id );
			
			return;
		}

		$term = wp_get_post_terms( $order_id, 'shop_order_status' );

		if ( !$term || is_wp_error( $term ) )
			return;

		$meta = get_option( 'taxonomy_' . $term[0]->term_id, false );

		if ( !$meta || !isset( $meta['woocommerce_wc_lite_orderstatus_email'] ) || '1' != $meta['woocommerce_wc_lite_orderstatus_email'] || !isset( $meta['woocommerce_wc_lite_orderstatus_message'] ) || '' == trim( $meta['woocommerce_wc_lite_orderstatus_message'] ) )
			return;

		if ( isset( $meta['woocommerce_wc_lite_orderstatus_send_to'] ) && '' != trim( $meta['woocommerce_wc_lite_orderstatus_send_to'] ) )
			$to = $meta['woocommerce_wc_lite_orderstatus_send_to'];
		else
			$to = get_post_meta( $order_id, '_billing_email', true );

		if ( !$to || '' == trim( $to ) )
			return;

		$message = $meta['woocommerce_wc_lite_orderstatus_message'];

		if ( !empty( $message ) ) 
			$message = stripslashes( $message );
			
		$message = $this->apply_code_filters( $message, $order_id );

		if ( isset( $meta['woocommerce_wc_lite_orderstatus_heading'] ) && '' != trim( $meta['woocommerce_wc_lite_orderstatus_heading'] ) )
			$heading = $meta['woocommerce_wc_lite_orderstatus_heading'];
		else
			$heading = '';

		$heading = $this->apply_code_filters( $heading, $order_id );

		if ( isset( $meta['woocommerce_wc_lite_orderstatus_subject'] ) && '' != trim( $meta['woocommerce_wc_lite_orderstatus_subject'] ) )
			$subject = $meta['woocommerce_wc_lite_orderstatus_subject'];
		else
			$subject = '';

		$subject = $this->apply_code_filters( $subject, $order_id );

		$headers = apply_filters('woocommerce_email_headers', '', 'rewards_message');

		$attachments = array();
		
		
		$mailer = $woocommerce->mailer();

		$message = $mailer->wrap_message( $heading, $message );
		
		$mailer->send( $to, $subject, $message, $headers, $attachments );

	}

	
	function do_status_change_alt( $order_id ) {
		global $woocommerce;

		$order = new WC_Order( $order_id );

		if ( empty( $order->post_status ) )
			return;

		$order->post_status = str_replace( 'wc-', '', $order->post_status );
		
		$statuses = $this->get_terms(); 
		
		$slug = false; 

		foreach( $statuses as $key => $vals ) {
		
			if ( $order->post_status == str_replace( 'wc-', '', $vals->slug ) )
				$slug = $vals->slug;
		
		}

		if ( empty( $slug ) )
			return; 
		
		$meta = get_option( 'az_custom_order_status_meta_' . $slug, false );

		if ( empty( $meta ) )
			return;

		if ( !$meta || !isset( $meta['woocommerce_wc_lite_orderstatus_email'] ) || '1' != $meta['woocommerce_wc_lite_orderstatus_email'] || !isset( $meta['woocommerce_wc_lite_orderstatus_message'] ) || '' == trim( $meta['woocommerce_wc_lite_orderstatus_message'] ) )
			return;

		if ( isset( $meta['woocommerce_wc_lite_orderstatus_send_to'] ) && '' != trim( $meta['woocommerce_wc_lite_orderstatus_send_to'] ) )
			$to = $meta['woocommerce_wc_lite_orderstatus_send_to'];
		else
			$to = get_post_meta( $order_id, '_billing_email', true );

		if ( !$to || '' == trim( $to ) )
			return;

		$message = $meta['woocommerce_wc_lite_orderstatus_message'];

		if ( !empty( $message ) ) 
			$message = stripslashes( $message );
			
		$message = $this->apply_code_filters( $message, $order_id );

		if ( isset( $meta['woocommerce_wc_lite_orderstatus_heading'] ) && '' != trim( $meta['woocommerce_wc_lite_orderstatus_heading'] ) )
			$heading = $meta['woocommerce_wc_lite_orderstatus_heading'];
		else
			$heading = '';

		$heading = $this->apply_code_filters( $heading, $order_id );

		if ( isset( $meta['woocommerce_wc_lite_orderstatus_subject'] ) && '' != trim( $meta['woocommerce_wc_lite_orderstatus_subject'] ) )
			$subject = $meta['woocommerce_wc_lite_orderstatus_subject'];
		else
			$subject = '';

		$subject = $this->apply_code_filters( $subject, $order_id );

		// Queue additional mail envelope headers
		$headers = apply_filters('woocommerce_email_headers', '', 'rewards_message');

		$attachments = array();
		
		//$attachments = apply_filters( 'order_status_manager_status_changed', $order_id );
		
		$mailer = $woocommerce->mailer();

		$message = $mailer->wrap_message( $heading, $message );

		$mailer->send( $to, $subject, $message, $headers, $attachments );

	}
	
	
	function apply_code_filters( $text, $order_id ) {

		if ( '' == trim( $text ) )
			return $text;

		$meta = get_post_custom( $order_id, true );

		$sitename = wp_specialchars_decode( get_option( 'blogname' ), ENT_QUOTES );

		$siteurl = wp_specialchars_decode( get_option( 'home' ), ENT_QUOTES );

		if ( false !== strpos( $text, '%o' ) ) { 
			$text = str_replace( '%o', $order_id, $text );

		}
		
		if ( false !== strpos( $text, '%bf' ) ) {

			$fname = $meta['_billing_first_name'][0];

			if ( !$fname || empty( $fname ) )
				$text = str_replace( '%bf', '', $text );
			else
				$text = str_replace( '%bf', $fname, $text );
		}

		if ( false !== strpos( $text, '%bl' ) ) {

			$lname = $meta['_billing_last_name'][0];

			if ( !$lname || empty( $lname ) )
				$text = str_replace( '%bl', '', $text );
			else
				$text = str_replace( '%bl', $lname, $text );
		}
		
		if ( false !== strpos( $text, '%sf' ) ) {

			$fname = $meta['_shipping_first_name'][0];

			if ( !$fname || empty( $fname ) )
				$text = str_replace( '%sf', '', $text );
			else
				$text = str_replace( '%sf', $fname, $text );
		}

		if ( false !== strpos( $text, '%sl' ) ) {

			$lname = $meta['_shipping_last_name'][0];

			if ( !$lname || empty( $lname ) )
				$text = str_replace( '%sl', '', $text );
			else
				$text = str_replace( '%sl', $lname, $text );
		}
		
		if ( false !== strpos( $text, '%s' ) )
			$text = str_replace( '%s', $sitename, $text );

		if ( false !== strpos( $text, '%u' ) )
			$text = str_replace( '%u', $siteurl, $text );

		if ( false !== strpos( $text, '%m' ) ) {
			$ma_id = woocommerce_get_page_id( 'myaccount' );
			$ma_url = get_permalink( $ma_id );
			$text = str_replace( '%m', $ma_url, $text );
		}

		return $text;
	}
	
	function bulk_admin_footer() {
		global $post_type;
		
		if ( !isset( $post_type ) )
			return;

		if ( 'shop_order' == $post_type ) {
		
			$terms = $this->get_terms( 'shop_order_status', array( 'hide_empty' => false ) );

			?>
			<script type="text/javascript">
			jQuery(document).ready(function() {
				
				<?php foreach( $terms as $t ) { ?>
				
				<?php if ( in_array( $t->slug, $this->defaults ) ) continue; ?>
				
				jQuery('<option>').val('mark_<?php echo $t->slug ?>').text('<?php _e( 'Mark ' . $t->slug, 'woocommerce' )?>').appendTo("select[name='action']");
				
				jQuery('<option>').val('mark_<?php echo $t->slug ?>').text('<?php _e( 'Mark ' . $t->slug, 'woocommerce' )?>').appendTo("select[name='action2']");

				
				<?php } ?>
			});
			</script>
			<?php
		}
	}


	/**
	* Process the new bulk actions for changing order status
	*
	* @access public
	* @return void
	*/
	function order_bulk_action() {
		global $post_type;

		if ( empty( $post_type ) )
			$post_type = isset( $_GET['post_type'] ) ?  $_GET['post_type'] : '';

		if ( 'shop_order' != $post_type )
			return;

		if ( !empty( $_GET['post_status'] ) && 'trash' == $_GET['post_status'] )
			return;

		if ( !empty( $_GET['action'] ) && 'trash' == $_GET['action'] )
			return;
			
		if ( !empty( $_GET['action'] ) ) {
		
			if ( false === strpos( $_GET['action'], 'mark_' ) )
				return;
		
		}

		$wp_list_table = _get_list_table( 'WP_Posts_List_Table' );
		
		$action = $wp_list_table->current_action();

		if ( empty( $action ) )
			return;

		$terms = $this->get_terms( 'shop_order_status', array( 'hide_empty' => false ) );
		
		if ( !$terms || is_wp_error( $terms ) )
			return;
			
		$new_status = null; 
		
		foreach( $terms as $t ) {
		
			if ( 'mark_' . $t->slug == $action ) { 
			
				$new_status = $t->slug;
				
				$report_action = 'marked_' . $t->slug;
				
				break;

			}
		}

		if ( empty( $new_status ) )
			return;
			
		$changed = 0;

		$post_ids = array_map( 'absint', (array) $_REQUEST['post'] );

		foreach( $post_ids as $post_id ) {
		
			$order = new WC_Order( $post_id );
			
			$order->update_status( $new_status, __( 'Order status changed by bulk edit:', 'woocommerce' ) );
			
			$changed++;
		}

		$sendback = add_query_arg( array( 'post_type' => 'shop_order', $report_action => $changed, 'ids' => join( ',', $post_ids ) ), '' );
		
		wp_redirect( $sendback );
		
		exit();
	}
	
	
	function order_bulk_admin_notices() {
		global $post_type, $pagenow;

		if ( !isset( $post_type ) )
			return;
		
		$terms = $this->get_terms( 'shop_order_status', array( 'hide_empty' => false ) );
		
		if ( !$terms || is_wp_error( $terms ) )
			return;
			
		foreach( $terms as $t ) { 
			
			if ( isset( $_REQUEST['marked_' . $t->slug ] ) ) {
			
				$number = isset( $_REQUEST['marked_'.$t->slug] ) ? absint( $_REQUEST['marked_' . $t->slug ] ) : '';

				if ( !$number )
					continue;
					
				if ( 'edit.php' == $pagenow && 'shop_order' == $post_type ) {
				
					$message = sprintf( _n( 'Order status changed.', '%s order statuses changed.', $number ), number_format_i18n( $number ) );
					
					echo '<div class="updated"><p>' . $message . '</p></div>';
					
				}
			}
		}
	}
}

